/*
 * DTSSampleCSAM.r
 * Copyright  1992-93 Apple Computer Inc. All Rights Reserved.
 */

#include "Types.r"
#include "SysTypes.r"
#include "OCE.r"
#include "DTSSampleCSAM.h"

/*
 * The MPW MakeFile defines Think_Rez as zero. This is used
 * to build either the final MPW-version of the CSAM or the
 * Think C project resource file for the Think version.
 * See ReadMeForThinkC.c for details.
 */
#ifndef Think_Rez
#define Think_Rez	1
#endif

/*
 * These definitions are needed for all resource components.
 */
 
/*
 *
 * kTemplateName		"DTS Sample CSAM" -- this is used
 *						to define resource names and the
 *						names of some strings. You should
 *						change it to something suitable for
 *						your application.
 * kIconResourceFile	"DTS Sample Icons" -- this is the path
 *						to a file containing the icon suite.
 *						This lets you edit icons in ResEdit.
 * kTemplateCodeFile	"::DTSSampleTemplate.code" -- the name of
 *						the file containing the template code resource.
 * kCSAMCreatorID		The creator id (registered with DTS) for your CSAM
 *						This is used in the C code to access the Audit
 *						logging record.
 *
 * The following definitions are needed for the 'vers' resource. They
 * may also be useful to configure code that is release-specific (for
 * example, you can configure test code with #if kReleaseStage == 'd'
 *
 * kMajorVersion		These four symbols are used to configure the
 * kMinorVersion		release number in the 'vers' resource. For
 * kReleaseStage		example "1.2d3" is defined by major = 1,
 * kPreReleaseNumber	minor = 2, stage = 'd', pre-release  = 3
 * kReleaseVersion		The release stage as a text string.
 * kCopyrightString		The applicable copyright string.
 */

/*
 * kCSAMCreatorID is the creator ID, registered with DTS, for the CSAM. Note that
 * it is also needed by the Makefile to set the actual file creator. Also, the Audit
 * library uses the creator id to define the logging record. Normally, the creator
 * id is defined by a -d operator in the Makefile's Rez command.
 *
 * kCSAMCreatorIDString is the creator ID as a string. This is needed to concatenate
 * the creator id onto certain strings (actually, RStrings) that must be unique among
 * all templates and resources.
 */
#ifndef kCSAMCreatorID
#define kCSAMCreatorID					'Pdsm'
#endif
#undef kCSAMCreatorID
#define kCSAMCreatorID					'Pdsm'

/*
 * This is your creator id as a normal string
 */
#ifndef Think_Rez
/*
 * This gets a syntax error on Think Rez
 */
#if kCSAMCreatorID != 'Pdsm'
		<< error, the following won't work >>
#endif
#endif
#define kCSAMCreatorIDString			"Pdsm"

/*
 * kCopyrightText is the standard copyright text.
 */
#define kCopyrightText					" 1992-93 Apple Computer Inc."

/*
 * Include the components that do the work.
 */
#include "DTSSampleCSAMDriver.r"
#include "DTSSampleCSAMTemplate.r"

/*
 * FREF/BNDL stuff
 */
resource 'BNDL' (
			kCSAMAspect + kDETAspectMainBitmap,
			kTemplateName,
			purgeable
		) {
	kCSAMCreatorID,
	0,							/* Version			*/
	{
		'FREF', { 0, kCSAMAspect + kDETAspectMainBitmap	},
		'ICN#', { 0, kCSAMAspect + kDETAspectMainBitmap	}
	}
};

resource 'FREF' (
			kCSAMAspect + kDETAspectMainBitmap,
			kTemplateName,
			purgeable
		) {
	'dash', 0, ""
};

type kCSAMCreatorID as 'STR ';

resource kCSAMCreatorID (0, "", purgeable) {
	kTemplateName " " kReleaseVersionString " " kCopyrightText
	", All Rights Reserved"
#if Think_Rez
	" (Think)"
#else
	" (MPW)"
#endif
	" ($$Date)"
};

resource 'vers' (1, "", purgeable) {
	kMajorVersion,
	kMinorVersion,
	kReleaseStage,
	kPreReleaseNumber,
	verUS,
	kReleaseVersionString,
	kReleaseVersionString " " kCopyrightText
#ifdef Think_Rez
		" (Think)"
#else
		$$format(" (%4d.%02d.%02d)", $$year, $$month, $$day)
#endif
};

resource 'vers' (2, "", purgeable) {
	kMajorVersion,
	kMinorVersion,
	kReleaseStage,
	kPreReleaseNumber,
	verUS,
	kReleaseVersionString,
	"For AOCE 1.0"
};

/*
 * Finder Get Info Resource (Inside Mac VI, 9-22)
 */
resource 'STR ' (-16397, "", purgeable) {
	"This document contains the \"" kTemplateName
	"\" Catalog Service Access Module (CSAM)."
	" It cannot be opened."
};
